@@ -19,6 +19,7 @@ from cryptography import fernet
 from oslo_config import cfg
 from oslo_log import log as logging
 from stevedore import driver as stevedore_driver
+from taskflow import retry
 from taskflow import task
 from taskflow.types import failure
 import tenacity
@@ -51,6 +52,26 @@ class BaseComputeTask(task.Task):
         self.rate_limit = amphora_rate_limit.AmphoraBuildRateLimit()
 
 
+class ComputeRetry(retry.Times):
+
+    def on_failure(self, history, *args, **kwargs):
+        last_errors = history[-1][1]
+        max_retry_attempt = CONF.controller_worker.amp_active_retries
+        for task_name, ex_info in last_errors.items():
+            if len(history) <= max_retry_attempt:
+                # When taskflow persistence is enabled and flow/task state is
+                # saved in the backend. If flow(task) is restored(restart of
+                # worker,etc) we are getting ex_info as None - we need to RETRY
+                # task to check its real state.
+                if ex_info is None or ex_info._exc_info is None:
+                    return retry.RETRY
+                excp = ex_info._exc_info[1]
+                if isinstance(excp, exceptions.ComputeWaitTimeoutException):
+                    return retry.RETRY
+
+        return retry.REVERT_ALL
+
+
 class ComputeCreate(BaseComputeTask):
     """Create the compute instance for a new amphora."""
 
@@ -127,8 +148,8 @@ class ComputeCreate(BaseComputeTask):
                 server_group_id=server_group_id,
                 availability_zone=amp_availability_zone)
 
-            LOG.debug("Server created with id: %s for amphora id: %s",
-                      compute_id, amphora_id)
+            LOG.info("Server created with id: %s for amphora id: %s",
+                     compute_id, amphora_id)
             return compute_id
 
         except Exception:
@@ -166,14 +187,14 @@ class CertComputeCreate(ComputeCreate):
 
         # load client certificate
         with open(CONF.controller_worker.client_ca,
-                  'r', encoding='utf-8') as client_ca:
+                  encoding='utf-8') as client_ca:
             ca = client_ca.read()
 
         key = utils.get_compatible_server_certs_key_passphrase()
         fer = fernet.Fernet(key)
         config_drive_files = {
             '/etc/octavia/certs/server.pem': fer.decrypt(
-                server_pem.encode("utf-8")),
+                server_pem.encode("utf-8")).decode("utf-8"),
             '/etc/octavia/certs/client_ca.pem': ca}
         return super().execute(
             amphora_id, config_drive_files=config_drive_files,
@@ -189,8 +210,10 @@ class DeleteAmphoraeOnLoadBalancer(BaseComputeTask):
     """
 
     def execute(self, loadbalancer):
-        db_lb = self.loadbalancer_repo.get(
-            db_apis.get_session(), id=loadbalancer[constants.LOADBALANCER_ID])
+        session = db_apis.get_session()
+        with session.begin():
+            db_lb = self.loadbalancer_repo.get(
+                session, id=loadbalancer[constants.LOADBALANCER_ID])
         for amp in db_lb.amphorae:
             # The compute driver will already handle NotFound
             try:
@@ -251,7 +274,7 @@ class ComputeDelete(BaseComputeTask):
                 raise
 
 
-class ComputeActiveWait(BaseComputeTask):
+class ComputeWait(BaseComputeTask):
     """Wait for the compute driver to mark the amphora active."""
 
     def execute(self, compute_id, amphora_id, availability_zone):
@@ -268,16 +291,16 @@ class ComputeActiveWait(BaseComputeTask):
             amp_network = availability_zone.get(constants.MANAGEMENT_NETWORK)
         else:
             amp_network = None
-        for i in range(CONF.controller_worker.amp_active_retries):
-            amp, fault = self.compute.get_amphora(compute_id, amp_network)
-            if amp.status == constants.ACTIVE:
-                if CONF.haproxy_amphora.build_rate_limit != -1:
-                    self.rate_limit.remove_from_build_req_queue(amphora_id)
-                return amp.to_dict()
-            if amp.status == constants.ERROR:
-                raise exceptions.ComputeBuildException(fault=fault)
-            time.sleep(CONF.controller_worker.amp_active_wait_sec)
 
+        amp, fault = self.compute.get_amphora(compute_id, amp_network)
+        if amp.status == constants.ACTIVE:
+            if CONF.haproxy_amphora.build_rate_limit != -1:
+                self.rate_limit.remove_from_build_req_queue(amphora_id)
+            return amp.to_dict()
+        if amp.status == constants.ERROR:
+            raise exceptions.ComputeBuildException(fault=fault)
+
+        time.sleep(CONF.controller_worker.amp_active_wait_sec)
         raise exceptions.ComputeWaitTimeoutException(id=compute_id)
 
 
@@ -319,8 +342,6 @@ class NovaServerGroupDelete(BaseComputeTask):
     def execute(self, server_group_id):
         if server_group_id is not None:
             self.compute.delete_server_group(server_group_id)
-        else:
-            return
 
 
 class AttachPort(BaseComputeTask):
