@@ -96,6 +96,32 @@ class TestLoadBalancerFlows(base.TestCase):
                           self.LBFlow.get_create_load_balancer_flow,
                           'BOGUS')
 
+    @mock.patch('octavia.common.rpc.NOTIFIER',
+                new_callable=MockNOTIFIER)
+    def test_get_create_load_balancer_flow_SRIOV(self, mock_get_net_driver,
+                                                 mock_notifier):
+        amp_flow = self.LBFlow.get_create_load_balancer_flow(
+            constants.TOPOLOGY_SINGLE, flavor_dict={constants.SRIOV_VIP: True})
+        self.assertIsInstance(amp_flow, flow.Flow)
+        self.assertIn(constants.AVAILABILITY_ZONE, amp_flow.requires)
+        self.assertIn(constants.BUILD_TYPE_PRIORITY, amp_flow.requires)
+        self.assertIn(constants.FLAVOR, amp_flow.requires)
+        self.assertIn(constants.LOADBALANCER_ID, amp_flow.requires)
+        self.assertIn(constants.SERVER_GROUP_ID, amp_flow.requires)
+        self.assertIn(constants.UPDATE_DICT, amp_flow.requires)
+        self.assertIn(constants.ADDITIONAL_VIPS, amp_flow.provides)
+        self.assertIn(constants.AMP_DATA, amp_flow.provides)
+        self.assertIn(constants.AMPHORA, amp_flow.provides)
+        self.assertIn(constants.AMPHORA_ID, amp_flow.provides)
+        self.assertIn(constants.AMPHORA_NETWORK_CONFIG, amp_flow.provides)
+        self.assertIn(constants.COMPUTE_ID, amp_flow.provides)
+        self.assertIn(constants.COMPUTE_OBJ, amp_flow.provides)
+        self.assertIn(constants.LOADBALANCER, amp_flow.provides)
+        self.assertIn(constants.PORT_DATA, amp_flow.provides)
+        self.assertIn(constants.SERVER_PEM, amp_flow.provides)
+        self.assertIn(constants.SUBNET, amp_flow.provides)
+        self.assertIn(constants.VIP, amp_flow.provides)
+
     @mock.patch('octavia.common.rpc.NOTIFIER',
                 new_callable=MockNOTIFIER)
     def test_get_delete_load_balancer_flow(self, mock_get_net_driver,
@@ -199,16 +225,17 @@ class TestLoadBalancerFlows(base.TestCase):
 
         self.assertIn(constants.LOADBALANCER_ID, amp_flow.requires)
         self.assertIn(constants.UPDATE_DICT, amp_flow.requires)
+        self.assertIn(constants.AMPHORA_ID, amp_flow.requires)
 
         self.assertIn(constants.AMPHORAE, amp_flow.provides)
+        self.assertIn(constants.AMPHORAE_STATUS, amp_flow.provides)
         self.assertIn(constants.AMP_VRRP_INT, amp_flow.provides)
         self.assertIn(constants.AMPHORAE_NETWORK_CONFIG, amp_flow.provides)
         self.assertIn(constants.LOADBALANCER, amp_flow.provides)
 
-        self.assertEqual(2, len(amp_flow.requires), amp_flow.requires)
-        self.assertEqual(4, len(amp_flow.provides), amp_flow.provides)
+        self.assertEqual(3, len(amp_flow.requires), amp_flow.requires)
+        self.assertEqual(5, len(amp_flow.provides), amp_flow.provides)
 
-        # Test mark_active=False
         amp_flow = self.LBFlow.get_post_lb_amp_association_flow(
             '123', constants.TOPOLOGY_ACTIVE_STANDBY)
 
@@ -216,14 +243,16 @@ class TestLoadBalancerFlows(base.TestCase):
 
         self.assertIn(constants.LOADBALANCER_ID, amp_flow.requires)
         self.assertIn(constants.UPDATE_DICT, amp_flow.requires)
+        self.assertIn(constants.AMPHORA_ID, amp_flow.requires)
 
         self.assertIn(constants.AMPHORAE, amp_flow.provides)
+        self.assertIn(constants.AMPHORAE_STATUS, amp_flow.provides)
         self.assertIn(constants.AMPHORAE_NETWORK_CONFIG, amp_flow.provides)
         self.assertIn(constants.AMP_VRRP_INT, amp_flow.provides)
         self.assertIn(constants.LOADBALANCER, amp_flow.provides)
 
-        self.assertEqual(2, len(amp_flow.requires), amp_flow.requires)
-        self.assertEqual(4, len(amp_flow.provides), amp_flow.provides)
+        self.assertEqual(3, len(amp_flow.requires), amp_flow.requires)
+        self.assertEqual(5, len(amp_flow.provides), amp_flow.provides)
 
     @mock.patch('octavia.common.rpc.NOTIFIER',
                 new_callable=MockNOTIFIER)
@@ -239,7 +268,6 @@ class TestLoadBalancerFlows(base.TestCase):
         self.assertIn(constants.UPDATE_DICT, create_flow.requires)
         self.assertIn(constants.BUILD_TYPE_PRIORITY, create_flow.requires)
         self.assertIn(constants.FLAVOR, create_flow.requires)
-        self.assertIn(constants.AMPHORAE_NETWORK_CONFIG, create_flow.requires)
         self.assertIn(constants.AVAILABILITY_ZONE, create_flow.requires)
         self.assertIn(constants.SERVER_GROUP_ID, create_flow.requires)
 
@@ -257,9 +285,10 @@ class TestLoadBalancerFlows(base.TestCase):
         self.assertIn(constants.SERVER_PEM, create_flow.provides)
         self.assertIn(constants.VIP, create_flow.provides)
         self.assertIn(constants.ADDITIONAL_VIPS, create_flow.provides)
+        self.assertIn(constants.AMPHORAE_NETWORK_CONFIG, create_flow.provides)
 
-        self.assertEqual(7, len(create_flow.requires))
-        self.assertEqual(14, len(create_flow.provides))
+        self.assertEqual(6, len(create_flow.requires))
+        self.assertEqual(15, len(create_flow.provides))
 
     @mock.patch('octavia.common.rpc.NOTIFIER',
                 new_callable=MockNOTIFIER)
@@ -286,6 +315,7 @@ class TestLoadBalancerFlows(base.TestCase):
         self.assertIn(constants.AMPHORA_ID, create_flow.provides)
         self.assertIn(constants.AMPHORA_NETWORK_CONFIG, create_flow.provides)
         self.assertIn(constants.AMPHORAE_NETWORK_CONFIG, create_flow.provides)
+        self.assertIn(constants.AMPHORAE_STATUS, create_flow.provides)
         self.assertIn(constants.COMPUTE_ID, create_flow.provides)
         self.assertIn(constants.COMPUTE_OBJ, create_flow.provides)
         self.assertIn(constants.DELTAS, create_flow.provides)
@@ -297,7 +327,7 @@ class TestLoadBalancerFlows(base.TestCase):
         self.assertIn(constants.ADDITIONAL_VIPS, create_flow.provides)
 
         self.assertEqual(6, len(create_flow.requires), create_flow.requires)
-        self.assertEqual(17, len(create_flow.provides),
+        self.assertEqual(18, len(create_flow.provides),
                          create_flow.provides)
 
     def _test_get_failover_LB_flow_single(self, amphorae):
@@ -329,10 +359,13 @@ class TestLoadBalancerFlows(base.TestCase):
         self.assertIn(constants.VIP, failover_flow.provides)
         self.assertIn(constants.ADDITIONAL_VIPS, failover_flow.provides)
         self.assertIn(constants.VIP_SG_ID, failover_flow.provides)
+        self.assertIn(constants.AMPHORA_FIREWALL_RULES, failover_flow.provides)
+        self.assertIn(constants.SUBNET, failover_flow.provides)
+        self.assertIn(constants.NEW_AMPHORAE, failover_flow.provides)
 
         self.assertEqual(6, len(failover_flow.requires),
                          failover_flow.requires)
-        self.assertEqual(13, len(failover_flow.provides),
+        self.assertEqual(17, len(failover_flow.provides),
                          failover_flow.provides)
 
     @mock.patch('octavia.common.rpc.NOTIFIER',
@@ -405,10 +438,14 @@ class TestLoadBalancerFlows(base.TestCase):
         self.assertIn(constants.VIP, failover_flow.provides)
         self.assertIn(constants.ADDITIONAL_VIPS, failover_flow.provides)
         self.assertIn(constants.VIP_SG_ID, failover_flow.provides)
+        self.assertIn(constants.SUBNET, failover_flow.provides)
+        self.assertIn(constants.AMPHORA_FIREWALL_RULES, failover_flow.provides)
+        self.assertIn(constants.SUBNET, failover_flow.provides)
+        self.assertIn(constants.NEW_AMPHORAE, failover_flow.provides)
 
         self.assertEqual(6, len(failover_flow.requires),
                          failover_flow.requires)
-        self.assertEqual(13, len(failover_flow.provides),
+        self.assertEqual(17, len(failover_flow.provides),
                          failover_flow.provides)
 
     @mock.patch('octavia.common.rpc.NOTIFIER',
